#include "dlgedit.h"
#include "kannadic.h"
#include <kconfig.h>
#include <qlayout.h>
#include <qstringlist.h>
#include <qlabel.h>

dlgEdit::dlgEdit( QWidget *parent ) : KDialogBase(KDialogBase::Plain, i18n("Edit"), Ok|Cancel, Ok, parent, "", true )
{
	QFrame* page = plainPage();
	QGridLayout *glay = new QGridLayout( page, 0, 0 );

	QLabel* lblFrom = new QLabel( i18n("From"), page );
	txtFrom = new KLineEdit( page );

	QLabel* lblTo = new QLabel( i18n("To"), page );
	txtTo = new KLineEdit( page );

	QLabel* lblType = new QLabel( i18n("Type"), page );
	cbxType = new KComboBox( page );
	cbxType->insertStringList( KannaDic::type() );

	glay->addWidget( lblFrom, 0, 0 );
	glay->addWidget( txtFrom, 0, 1 );
	glay->addWidget( lblTo, 1, 0 );
	glay->addWidget( txtTo, 1, 1 );
	glay->addWidget( lblType, 2, 0 );
	glay->addWidget( cbxType, 2, 1 );
	glay->activate();

	KConfig *conf = KGlobal::config();
	conf->setGroup("dlgEdit");
	if( conf->readNumEntry( "width", width() ) != width() || conf->readNumEntry( "height", height() ) != height() ){
		resize( conf->readNumEntry( "width", width() ), conf->readNumEntry( "height", height() ) );
	}
}

dlgEdit::~dlgEdit()
{
	KConfig *conf = KGlobal::config();
	conf->setGroup("dlgEdit");
	conf->writeEntry( "width", width() );
	conf->writeEntry( "height", height() );
}

void dlgEdit::setEditMode( EditModes mode )
{
	switch( mode )
	{
		case APPEND:
			setIcon( BarIcon( "filenew" ) );
			setCaption( i18n("Append") );
			txtFrom->setText( "" );
			txtTo->setText( "" );
			cbxType->setCurrentItem( 0 );
			break;
		case EDIT:
			setIcon( BarIcon( "edit" ) );
			setCaption( i18n("Edit") );
	}
	txtFrom->setFocus();
}

QString dlgEdit::from()
{
	return txtFrom->text();
}

void dlgEdit::setFrom( QString from )
{
	txtFrom->setText( from );
}

QString dlgEdit::to()
{
	return txtTo->text();
}

void dlgEdit::setTo( QString to )
{
	txtTo->setText( to );
}

QString dlgEdit::type()
{
	return cbxType->currentText();
}

void dlgEdit::setType( QString type )
{
	cbxType->setCurrentText( type );
}
