#include "dlgexporter.h"

#include <qstring.h>
#include <qtextcodec.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <klocale.h>
#include <kdebug.h>
#include <kurl.h>
#include <kmessagebox.h>
#include <qapplication.h>
#include <qdom.h>
#include <kuniqueapplication.h>
#include <kaboutdata.h>
#include <qprogressdialog.h>
#include <qcursor.h>
#include <kiconloader.h>
#include <kio/netaccess.h>
#include <ktempfile.h>

bool Exporter::exportKannadicXMLDictionary( QWidget* parent, KListView* lvwDictionary, KURL a_url )
{
	KURL url;
	QString fileName;
	bool bllocal = true;
	if( a_url.isEmpty() ){
		KConfig *conf = KGlobal::config();
		conf->setGroup("dlgExporter");
		QString startdir = conf->readEntry( "StartDir", QString::null );

		url = KFileDialog::getSaveURL( startdir
			, i18n("*.ktd|Kannadic XML dictionary files(*.ktd)\n*.xml|XML files(*.xml)\n*.*|All files"), parent );

		if( url.isEmpty() ) return false;

		bllocal = url.isLocalFile();
		if( bllocal ){
			conf->writeEntry( "StartDir", url.directory() );
			fileName = url.path();
		}else{
			KTempFile tmpFile;
			tmpFile.setAutoDelete( true );
			fileName = tmpFile.name();
		}
	}else{
		fileName = a_url.path();
	}

	if( !fileName.isEmpty() ){
		QFile file( fileName );
		if ( !file.open( IO_WriteOnly ) ){
			KMessageBox::sorry( parent, i18n("KannaDic can not open the file.") );
			return false;
		}
		const KAboutData* aboutData = KUniqueApplication::kApplication()->aboutData();
		QDomDocument doc( aboutData->appName() );
		doc.appendChild( doc.createProcessingInstruction( "xml", "version=\"1.0\" encoding=\"UTF-8\"" ) );

		QDomElement editor = doc.createElement( aboutData->appName() );

		KConfig emailConf( QString::fromLatin1("emaildefaults") );
		// find out the default profile
		emailConf.setGroup( QString::fromLatin1("Defaults") );
		QString profile = QString::fromLatin1("PROFILE_");
		profile += emailConf.readEntry( QString::fromLatin1("Profile"), QString::fromLatin1("Default") );
		emailConf.setGroup( profile );

		editor.setAttribute( "author", emailConf.readEntry( QString::fromLatin1("FullName") ) );
		editor.setAttribute( "email", emailConf.readEntry( QString::fromLatin1("EmailAddress") ) );
		editor.setAttribute( "editor", QString("%1(%2)").arg( aboutData->programName() ).arg( aboutData->version() ) );
		editor.setAttribute( "mine", "x-application/kannadic" );
		doc.appendChild( editor );

		QApplication::setOverrideCursor( waitCursor );
		QProgressDialog* progress;
		if( a_url.isEmpty() ){
			progress = new QProgressDialog( parent );
			progress->setIcon( BarIcon("fileexport") );
			progress->setCaption( i18n("Export" ) );
			progress->setLabelText( i18n( "Please wait..." ) );
			progress->setCursor( QCursor::WaitCursor );
			progress->setCancelButton( 0 );
			progress->setTotalSteps( lvwDictionary->childCount() - 1 );
			progress->show();
		}
		QListViewItemIterator it( lvwDictionary );
		while ( it.current() ) {
			QDomElement e = doc.createElement( "word" );
			e.setAttribute( "from", it.current()->text( 0 ) );
			e.setAttribute( "to", it.current()->text( 1 ) );
			e.setAttribute( "type", it.current()->text( 2 ) );
			editor.appendChild( e );
			if( a_url.isEmpty() ){
				progress->setProgress( progress->progress() + 1 );
			}
			++it;
		}
		if( a_url.isEmpty() ){
			delete progress;
		}

		QApplication::restoreOverrideCursor();

		QTextStream str( &file );
		str.setCodec( QTextCodec::codecForName("eucJP") );
		str << doc;
		file.close();
	}else{
		return false;
	}
	if( !bllocal ){
		if( !KIO::NetAccess::upload( fileName, url ) ) {
			KMessageBox::error( parent, i18n( "Could not save remote file" ) );
			return false;
		}
	}
	return true;
}
