#include "dlgimporter.h"

#include <qstring.h>
#include <qtextcodec.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <klocale.h>
#include <kdebug.h>
#include <kurl.h>
#include <kmessagebox.h>
#include <qapplication.h>
#include <qdom.h>
#include <kuniqueapplication.h>
#include <kaboutdata.h>
#include <kio/netaccess.h>

QValueList<Word> Importer::importCannaTextDictionary( QWidget* parent )
{
	QValueList<Word> list;

	KConfig *conf = KGlobal::config();
	conf->setGroup("dlgImporter");
	QString startdir = conf->readEntry( "StartDir", QString::null );

	KURL url = KFileDialog::getOpenURL( startdir
		, i18n("*.ctd *.t|Canna text dictionary files(*.ctd *.t)\n*.txt|Text files(*.txt)\n*.*|All files"), parent );

	if( url.isEmpty() ) return list;
	QString fileName;
	bool blLocal = url.isLocalFile();
	if( blLocal ){
		fileName = url.path();
		conf->writeEntry( "StartDir", url.directory() );
	}else{
		if( !KIO::NetAccess::download( url, fileName ) ){
			KMessageBox::sorry( parent, KIO::NetAccess::lastErrorString() );
			return list;
		}
	}

	QTextCodec* codec = QTextCodec::codecForName("eucJP");

	if( !fileName.isEmpty() ){
		QFile file( fileName );
		if ( !file.open( IO_ReadOnly ) ){
			KMessageBox::sorry( parent, i18n("KannaDic can not open the file.") );
			return list;
		}
		QString comment;
		QTextStream ts( &file );
		QApplication::setOverrideCursor( waitCursor );
		while( !ts.atEnd() ){
			QString str( ts.readLine() );
			if( str.isEmpty() ){
				continue;
			}else if( str.at(0) == '#' ){
				comment += str;
				comment += "\n";
				continue;
			}
			str = codec->fromUnicode( str );
			QTextStream line( str, IO_ReadOnly );
			Word word;
			line >> word.from;
			word.from = codec->toUnicode( KannaDic::unescape(word.from) );
			while( !line.atEnd() ){
				QString temp, type;
				line >> temp;
				if( temp.at(0) == '#' ){
					int asterisc = temp.find( '*' );
					if( asterisc != -1 ){
						temp = temp.left( asterisc );
					}
					type = KannaDic::code2name( KannaDic::unescape(temp) );
					if( type == QString::null ){
						kdDebug() << temp << endl;
					}
					word.type = codec->toUnicode( type );
				}else{
					word.to = codec->toUnicode( KannaDic::unescape( temp ) );
//					kdDebug() << word.from << "\t" << word.to << endl;
					list.push_back( word );
				}
			}
		}
		file.close();
		QApplication::restoreOverrideCursor();
		if( !comment.isEmpty() ){
			KMessageBox::information( parent, comment, i18n("Exclude line(s)"), QString::null, KMessageBox::AllowLink );
		}
	}
	if( !blLocal ){
		KIO::NetAccess::removeTempFile( fileName );
	}
	return list;
}

QValueList<Word> Importer::importWindowsTextDictionary( QWidget* parent )
{
	QValueList<Word> list;

	KConfig *conf = KGlobal::config();
	conf->setGroup("dlgImporter");
	QString startdir = conf->readEntry( "StartDir", QString::null );

	KURL url = KFileDialog::getOpenURL( startdir
		, i18n("*.txt|Microsoft IME text dictionary files(*.txt)\n*.*|All files"), parent );

	if( url.isEmpty() ) return list;

	QString fileName;
	bool blLocal = url.isLocalFile();
	if( blLocal ){
		fileName = url.path();
		conf->writeEntry( "StartDir", url.directory() );
	}else{
		if( !KIO::NetAccess::download( url, fileName ) ){
			KMessageBox::sorry( parent, KIO::NetAccess::lastErrorString() );
			return list;
		}
	}

	QTextCodec* codecEuc = QTextCodec::codecForName("eucJP");
	QTextCodec* codecSjis = QTextCodec::codecForName("Shift-JIS");

	if( !fileName.isEmpty() ){
		QFile file( fileName );
		if ( !file.open( IO_ReadOnly ) ){
			KMessageBox::sorry( parent, i18n("KannaDic can not open the file.") );
			return list;
		}
		QString comment;
		QTextStream ts( &file );
		ts.setCodec( codecSjis );
		QApplication::setOverrideCursor( waitCursor );
		while( !ts.atEnd() ){
			QString str( ts.readLine() );
			if( str.isEmpty() ){
				continue;
			}else if( str.at(0) == '!' ){
				comment += str;
				comment += "\n";
				continue;
			}
			Word word;
			QStringList temp = QStringList::split( "\t", str );
			word.from = temp[0];
			word.to = temp[1];
			word.type = temp[2];
			word.type = codecEuc->fromUnicode( word.type );
			word.type = win2canna( word.type );
			word.type = codecEuc->toUnicode( word.type );
			list.push_back( word );
		}
		file.close();
		QApplication::restoreOverrideCursor();
		if( !comment.isEmpty() ){
			KMessageBox::information( parent, comment, i18n("Exclude line(s)"), QString::null, KMessageBox::AllowLink );
		}
	}
	if( !blLocal ){
		KIO::NetAccess::removeTempFile( fileName );
	}

	return list;
}

QValueList<Word> Importer::importKannadicXMLDictionary( QWidget* parent, KURL a_url )
{
	QValueList<Word> list;
	QString fileName;
	bool blLocal = true;
	KURL url;

	KConfig *conf = KGlobal::config();
	if( a_url.isEmpty() ){
		conf->setGroup("dlgImporter");
		QString startdir = conf->readEntry( "StartDir", QString::null );

		url = KFileDialog::getOpenURL( startdir
			, i18n("*.ktd|Kannadic XML dictionary files(*.ktd)\n*.xml|XML files(*.xml)\n*.*|All files"), parent );
		if( url.isEmpty() ) return list;
		blLocal = url.isLocalFile();
		if( blLocal ){
			fileName = url.path();
			conf->writeEntry( "StartDir", url.directory() );
		}else{
			if( !KIO::NetAccess::download( url, fileName ) ){
				KMessageBox::sorry( parent, KIO::NetAccess::lastErrorString() );
				return list;
			}
		}
	}else{
		fileName = a_url.path();
	}

	QTextCodec* codec = QTextCodec::codecForName("eucJP");

	if( !fileName.isEmpty() ){
		QFile file( fileName );
		if ( !file.open( IO_ReadOnly ) ){
			KMessageBox::sorry( parent, i18n("KannaDic can not open the file.") );
			return list;
		}

		QString comment;
		QApplication::setOverrideCursor( waitCursor );

		QDomDocument* doc = new QDomDocument();
		if( !doc->setContent( &file ) ){
			KMessageBox::sorry( parent, i18n("KannaDic can not parse this file.") );
			return list;
		}
		const KAboutData* aboutData = KUniqueApplication::kApplication()->aboutData();
		if( doc->doctype().name() != aboutData->appName() ){
			KMessageBox::sorry( parent, i18n("KannaDic can not parse this file.") );
			return list;
		}

		QDomElement editor = doc->documentElement();
		comment += i18n( QString( "author:%1\n" ).arg( editor.attribute("author") ) );
		comment += i18n( QString( "email:%1\n" ).arg( editor.attribute("email") ) );
		QDomNode child = editor.firstChild();
		Word word;
		while( !child.isNull() ){
			QDomElement elem = child.toElement();
			word.from = elem.attribute("from");
			word.to = elem.attribute("to");
			word.type = elem.attribute("type");
			list.push_back( word );
			child = child.nextSibling();
		}
		file.close();
		QApplication::restoreOverrideCursor();
		if( a_url.isEmpty() && !comment.isEmpty() ){
			KMessageBox::information( parent, comment, i18n("Exclude line(s)"), QString::null, KMessageBox::AllowLink );
		}
	}
	if( !blLocal ){
		KIO::NetAccess::removeTempFile( fileName );
	}
	return list;
}

QString Importer::win2canna( QString from )
{
	static QMap<QString, QString> table;
	if( table.empty() ){
		int count = sizeof(trans_ime2canna)/sizeof(struct Trans);
		for( int i = 0; i != count; i++ ){
			table[trans_ime2canna[i].from] = KannaDic::code2name( trans_ime2canna[i].to );
		}
	}
	return table[from];
}

QString Importer::win2cannax( QString from )
{
	int count = sizeof(trans_ime2canna)/sizeof(struct Trans);
	for( int i = 0; i != count; i++ ){
		if( trans_ime2canna[i].from == from ){
			return trans_ime2canna[i].to;
		}
	}
	return QString::null;
}
