#include "dlgpreferences.h"
#include <kconfig.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <klocale.h>
#include <qlayout.h>
#include <qlabel.h>

dlgPreferences::dlgPreferences( QWidget *parent )
	: KDialogBase(KDialogBase::Plain, i18n("Configure"), Default|Ok|Cancel, Ok, parent, "", true )
{
	QFrame* page = plainPage();
	QGridLayout *glay = new QGridLayout( page, 0, 0 );

	btnColor = new KColorButton( page );
	connect( btnColor, SIGNAL( changed( const QColor& ) ), this, SLOT( slotEnableButtonOK() ) );
	QLabel* lblAltBGColor = new QLabel( btnColor, i18n("AlternateBackgroundColor"), page );

	chkSaveClipboard = new QCheckBox( i18n( "Save Clipboard" ), page );
	connect( chkSaveClipboard, SIGNAL( clicked() ), this, SLOT( slotEnableButtonOK() ) );

	glay->addWidget( lblAltBGColor, 0, 0 );
	glay->addWidget( btnColor, 0, 1 );
	glay->addMultiCellWidget( chkSaveClipboard, 1, 1, 0, 1 );
	glay->activate();

	setIcon( BarIcon( "configure" ) );
	KConfig *conf = KGlobal::config();
	conf->setGroup( "dlgPreferences" );
	if( conf->readNumEntry( "width", width() ) != width() || conf->readNumEntry( "height", height() ) != height() ){
		resize( conf->readNumEntry( "width", width() ), conf->readNumEntry( "height", height() ) );
	}
}

dlgPreferences::~dlgPreferences()
{
	KConfig *conf = KGlobal::config();
	conf->setGroup( "dlgPreferences" );
	conf->writeEntry( "width", width() );
	conf->writeEntry( "height", height() );
}

QColor dlgPreferences::color()
{
	return btnColor->color();
}

void dlgPreferences::setColor( QColor color )
{
	btnColor->setColor( color );
	enableButtonOK( false );
}

bool dlgPreferences::saveClipboard()
{
	return chkSaveClipboard->isChecked();
}

void dlgPreferences::setSaveClipboard( bool saveClipboard )
{
	chkSaveClipboard->setChecked( saveClipboard );
}

void dlgPreferences::slotEnableButtonOK()
{
	enableButtonOK( true );
}

void dlgPreferences::slotDefault()
{
	btnColor->setColor( lightGray );
	chkSaveClipboard->setChecked( false );
	slotEnableButtonOK();
}
