#include "dlgopendic.h"
#include "dlgpreferences.h"
#include "frmmain.h"
#include "dlgimporter.h"
#include "dlgexporter.h"

#include <kapplication.h>
#include <kaccel.h>
#include <kaction.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <kmenubar.h>
#include <kmessagebox.h>
#include <kpopupmenu.h>
#include <kstatusbar.h>
#include <qcursor.h>
#include <qheader.h>
#include <qpopupmenu.h>
#include <qprogressdialog.h>
#include <qptrlist.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qgrid.h>
#include <qlayout.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qclipboard.h>
#include <kxmlguifactory.h>
#include <kstandarddirs.h>
#include <klineeditdlg.h>
#include <qtextcodec.h>
#include <kstdguiitem.h>

frmMain::frmMain()
{
	m_changed.clear();
	m_changed_index = m_changed.size() - 1;
	m_before.clear();
	m_after.clear();
	m_blDicOpened = false;
	m_blActivate = 0;
	m_dicname = QString::null;

	setupCentralWidget();
	setupAcitons();
	setStandardToolBarMenuEnabled( true );

	setupStatusBar();
	setEditEnabled( false );

	readConfiguration();
	m_blActivate = 1;
}

frmMain::~frmMain()
{
	writeConfiguration();
}

void frmMain::setupCentralWidget()
{
	tabBase = new QTabWidget( this );
	connect( tabBase, SIGNAL( currentChanged( QWidget* ) ), this, SLOT( slotTabChanged() ) );
	setupWordTab();
	setupListTab();
	setCentralWidget( tabBase );
}

void frmMain::slotTabChanged()
{
	if( m_blActivate == 0 ){
		return;
	}
	KConfig *conf = KGlobal::config();
	switch( tabBase->currentPageIndex() )
	{
		case TAB_WORD:
			if( m_blActivate == 2 ){
				saveMainWindowSettings( conf, "modeList" );
			}
			m_blActivate = 2;
			createGUI( "kannadic_word.rc" );
			applyMainWindowSettings( conf, "modeWord" );
			statusBar()->hide();
			break;
		case TAB_LIST:
			if( !m_dicname.isEmpty() && !m_blDicOpened ){
				dicOpen( m_dicname );
				m_blDicOpened = true;
			}
			if( m_blActivate == 2 ){
				saveMainWindowSettings( conf, "modeWord" );
			}
			m_blActivate = 2;
			createGUI( "kannadic_list.rc" );
			applyMainWindowSettings( conf, "modeList" );
			break;
	}
}

void frmMain::setupWordTab()
{
	QGrid *grdWord = new QGrid( 2, this );
	tabBase->addTab( grdWord, SmallIcon("document"), i18n("Regist Word") );
	grdWord->setSpacing( 0 );
	QLabel* lblFrom = new QLabel( i18n("From"), grdWord );
	txtFrom = new KLineEdit( grdWord );
	lblFrom->setBuddy( txtFrom );

	QLabel* lblTo = new QLabel( i18n("To"), grdWord );
	txtTo = new KLineEdit( grdWord );
	lblTo->setBuddy( txtTo );

	QLabel* lblType = new QLabel( i18n("Type"), grdWord );
	cbxType = new KComboBox( grdWord );
	cbxType->insertStringList( KannaDic::type() );
	lblType->setBuddy( cbxType );

	(new QWidget( grdWord ))->setMinimumHeight(0);
	(new QWidget( grdWord ))->setMinimumHeight(0);

	new QWidget( grdWord );
	QHBox* hbxWord = new QHBox( grdWord );
	hbxWord->setMargin( 10 );
	hbxWord->setSpacing( 10 );

	new QWidget( hbxWord );
	btnOK = new KPushButton( KStdGuiItem::ok(), hbxWord );
	btnOK->setDefault( true );
	btnOK->setFixedSize( btnOK->sizeHint() );
	connect( btnOK, SIGNAL( clicked() ), this, SLOT( slotSave() ) );

	KPushButton* btnCancel = new KPushButton( KStdGuiItem::cancel(), hbxWord );
	btnCancel->setFixedSize( btnCancel->sizeHint() );
	connect( btnCancel, SIGNAL( clicked() ), this, SLOT( close() ) );

	hbxWord->setFixedHeight( hbxWord->sizeHint().height() );

	if( KApplication::clipboard()->text( QClipboard::Clipboard ) != "" ){
		txtTo->setText( KApplication::clipboard()->text( QClipboard::Clipboard ) );
	}else if( KApplication::clipboard()->text( QClipboard::Clipboard ) != "" ){
		txtTo->setText( KApplication::clipboard()->text( QClipboard::Selection ) );
	}
}

void frmMain::setupListTab()
{
	splList = new QSplitter( Qt::Vertical, this );
	splList->setOpaqueResize( true );
	tabBase->addTab( splList, SmallIcon("view_detailed"), i18n("Dictionary Tool") );

	lvwDictionary = new KListView( splList );
	clipboard = new QVBox( splList );
	QHBox* hbxClipboard = new QHBox( clipboard );
	QLabel* label = new QLabel( i18n("Clipboard"), hbxClipboard );
	label->setPixmap( SmallIcon("klipper") );
	label->setMinimumWidth( label->height() );
	label->setMaximumWidth( label->height() );
	label = new QLabel( i18n("Clipboard"), hbxClipboard );

	QPushButton* button = new QPushButton( SmallIcon("remove"), "", hbxClipboard );
	button->setMinimumWidth( button->height() );
	button->setMaximumWidth( button->height() );
	button->setFlat( true );
	connect( button, SIGNAL( clicked() ), this, SLOT( slotClipboardHide() ) );
	lvwClipboard = new KListView( clipboard );

	lvwDictionary->setAllColumnsShowFocus( true );
	lvwDictionary->setSelectionModeExt( KListView::Extended );

	lvwDictionary->addColumn( i18n("From") );
	lvwDictionary->addColumn( i18n("To") );
	lvwDictionary->addColumn( i18n("Type") );

	lvwClipboard->setAllColumnsShowFocus( true );
	lvwClipboard->setSelectionModeExt( KListView::Extended );

	lvwClipboard->addColumn( i18n("From") );
	lvwClipboard->addColumn( i18n("To") );
	lvwClipboard->addColumn( i18n("Type") );

	for( int i = 0; i != lvwDictionary->header()->count(); i++ ){
		lvwDictionary->setColumnWidthMode( i, QListView::Manual );
		lvwClipboard->setColumnWidthMode( i, QListView::Manual );
	}

	connect( lvwDictionary, SIGNAL( executed( QListViewItem* ) ), this, SLOT( slotEdit() ) );
	connect( lvwDictionary, SIGNAL( returnPressed( QListViewItem* ) ), this, SLOT( slotEdit() ) );
	connect( lvwDictionary, SIGNAL( rightButtonPressed ( QListViewItem *, const QPoint &, int ) ), this, SLOT( slotPopup( QListViewItem *, const QPoint &, int ) ) );
	/*
	lvwDictionary->setAcceptDrops( true );
	lvwDictionary->setDragEnabled( true );
	lvwClipboard->setDragEnabled( true );
	*/
	//connect( lvwDictionary, SIGNAL( dropped( KListView*, QDropEvent*, QListViewItem* ) ), this, SLOT( slotDictionaryDropped( KListView*, QDropEvent*, QListViewItem* ) ) );
}

void frmMain::readConfiguration()
{
	KConfig *conf = KGlobal::config();
	conf->setGroup("frmMain");
	splList->setSizes( conf->readIntListEntry( "Split" ) );
	slotShowClipboard( conf->readBoolEntry( "ShowClipboard", false ) );
	lvwDictionary->setAlternateBackground( conf->readColorEntry( "AlternateBackground", &lightGray ) );
	lvwClipboard->setAlternateBackground( conf->readColorEntry( "AlternateBackground", &lightGray ) );
	tabBase->setCurrentPage( conf->readNumEntry( "TabIndex", TAB_WORD ) );
	m_saveClipboard = conf->readBoolEntry( "SaveClipboard", false );
	if( m_saveClipboard ){
		loadClipboard();
	}

	m_dicname = conf->readEntry( "dicname", QString::null );
	if( !m_dicname.isEmpty() ){
		slotDicOpen( m_dicname );
	}
	lvwDictionary->restoreLayout( conf, "lvwDictionary" );
	lvwClipboard->restoreLayout( conf, "lvwClipboard" );
}

void frmMain::writeConfiguration()
{
	KConfig *conf = KGlobal::config();
	conf->setGroup("frmMain");
	conf->writeEntry( "dicname", m_dicname );
	conf->writeEntry( "TabIndex", tabBase->currentPageIndex() );
	conf->writeEntry( "Split", splList->sizes() );
	lvwDictionary->saveLayout( conf, "lvwDictionary" );
	lvwClipboard->saveLayout( conf, "lvwClipboard" );
	conf->writeEntry( "AlternateBackground", lvwDictionary->alternateBackground() );
	conf->writeEntry( "ShowClipboard", actShowClipboard->isChecked() );

	conf->writeEntry( "SaveClipboard", m_saveClipboard );
	if( m_saveClipboard ){
		saveClipboard();
	}
	switch( tabBase->currentPageIndex() )
	{
		case TAB_WORD:
			saveMainWindowSettings( conf, "modeWord" );
			break;
		case TAB_LIST:
			saveMainWindowSettings( conf, "modeList" );
			break;
	}
}

void frmMain::setupAcitons()
{
	KAction *act;
	KActionMenu* actMenu;

	QString strDic( i18n( "&Dictionary" ) );

	act = new KAction( i18n("&Create..."), "folder_new", KShortcut(CTRL+Qt::Key_N), this, SLOT( slotDicNew() ), actionCollection(), "createdic" );
	act->setWhatsThis( i18n("Create new user dictionary.") );

	act = KStdAction::open( this, SLOT( slotDicOpen() ), actionCollection(), "open" );
	act->setWhatsThis( i18n("Open a mounting dictionary for editing.") );

	act = new KAction( i18n("&Delete..."), "cancel", KShortcut(CTRL+Qt::Key_D), this, SLOT( slotDicDel() ), actionCollection(), "deletedic" );
	act->setWhatsThis( i18n( "Delete current user dictionary." ) );
	actEdit.append( act );

	act = new KAction( i18n("&Save"), "filesave", KShortcut( Qt::Key_Return ), this, SLOT( slotSave() ), actionCollection(), "word_save" );
	act->setWhatsThis( i18n( "Save word." ) );
	actEdit.append( act );

	act = KStdAction::save( this, SLOT( slotDicSave() ), actionCollection(), "save" );
	act->setWhatsThis( i18n( "Save modified." ) );
	actEdit.append( act );

	act = KStdAction::quit( this, SLOT( close() ), actionCollection(), "quit" );
	act->setWhatsThis( i18n( "Close this window." ) );


	act = new KAction( i18n("&Append..."), "filenew", KShortcut(Qt::Key_Insert), this, SLOT( slotAppend() ), actionCollection(), "append" );
	act->setWhatsThis( i18n( "Append new word." ) );
	actEdit.append( act );

	act = new KAction( i18n("&Edit..."), "edit", KShortcut(Qt::Key_Return), this, SLOT( slotEdit() ), actionCollection(), "edit" );
	act->setWhatsThis( i18n( "Edit selected word(s)." ) );
	actEdit.append( act );

	act = new KAction( i18n("&Delete"), "editdelete", KShortcut(Qt::Key_Delete), this, SLOT( slotDelete() ), actionCollection(), "delete" );
	act->setWhatsThis( i18n( "Delete selected word(s)." ) );
	actEdit.append( act );

	act = KStdAction::cut( this, SLOT( slotCut() ), actionCollection(), "cut" );
	act->setWhatsThis( i18n( "Cut to Clipboard." ) );
	actEdit.append( act );

	act = KStdAction::copy( this, SLOT( slotCopy() ), actionCollection(), "copy" );
	act->setWhatsThis( i18n( "Copy to Clipboard." ) );
	actEdit.append( act );

	act = KStdAction::paste( this, SLOT( slotPaste() ), actionCollection(), "paste" );
	act->setWhatsThis( i18n( "Paste from Clipboard." ) );
	actEdit.append( act );

	act = KStdAction::selectAll( this, SLOT( slotSelectAll() ), actionCollection(), "selectall" );
	act->setWhatsThis( i18n( "Select all words." ) );
	actEdit.append( act );

	act = KStdAction::deselect( this, SLOT( slotDeselect() ), actionCollection(), "deselect" );
	act->setWhatsThis( i18n( "Deselect all words." ) );
	actEdit.append( act );

	actUndo = KStdAction::undo( this, SLOT( slotUndo() ), actionCollection(), "undo" );
	actUndo->setWhatsThis( i18n( "Undo a recent work." ) );

	actRedo = KStdAction::redo( this, SLOT( slotRedo() ), actionCollection(), "redo" );
	actRedo->setWhatsThis( i18n( "Redo a recent work." ) );

	actImport = new KActionMenu( i18n("&Import"), "fileimport", actionCollection(), "import" );
	act = new KAction( i18n("&Canna text dictionary..."), "kannadic", KShortcut(), this, SLOT( slotImportCannaTextDictionary() ), actionCollection() );
	act->setWhatsThis( i18n( "Import canna text dictionary." ) );
	actImport->insert( act );

	act = new KAction( i18n("&MS-IME text dictionary..."), "kannadic", KShortcut(), this, SLOT( slotImportWindowsTextDictionary() ), actionCollection() );
	act->setWhatsThis( i18n( "Import Microsoft IME text dictionary." ) );
	actImport->insert( act );

	act = new KAction( i18n("&KannaDic XML dictionary..."), "kannadic", KShortcut(), this, SLOT( slotImportKannadicXMLDictionary() ), actionCollection() );
	act->setWhatsThis( i18n( "Import KannaDic XML dictionary." ) );
	actImport->insert( act );

	actExport = new KActionMenu( i18n("&Export"), "fileexport", actionCollection(), "export" );
	act = new KAction( i18n("&KannaDic XML dictionary..."), "kannadic", KShortcut(), this, SLOT( slotExportKannadicXMLDictionary() ), actionCollection() );
	act->setWhatsThis( i18n( "Export KannaDic XML dictionary." ) );
	actExport->insert( act );

	actShowClipboard = new KToggleAction( i18n("Show &Clipboard"), "klipper", KShortcut(), actionCollection(), "showclipboard" );
	connect( actShowClipboard, SIGNAL( toggled( bool ) ), this, SLOT( slotShowClipboard( bool ) ) );

	actShowMenubar = KStdAction::showMenubar( NULL, NULL, actionCollection(), "showmenubar" );
	actShowMenubar->setWhatsThis( i18n( "Toggle menubar's visible." ) );
	connect( actShowMenubar, SIGNAL( toggled( bool ) ), this, SLOT( slotShowMenubar( bool ) ) );

	actShowToolbar = KStdAction::showToolbar( NULL, NULL, actionCollection(), "showtoolbar" );
	actShowToolbar->setWhatsThis( i18n( "Toggle toolbar's visible." ) );
	connect( actShowToolbar, SIGNAL( toggled( bool ) ), this, SLOT( slotShowToolbar( bool ) ) );

	actShowStatusbar = KStdAction::showStatusbar( NULL, NULL, actionCollection(), "showstatusbar" );
	actShowStatusbar->setWhatsThis( i18n( "Toggle statusbar's visible." ) );
	connect( actShowStatusbar, SIGNAL( toggled( bool ) ), this, SLOT( slotShowStatusbar( bool ) ) );

	act = KStdAction::preferences( this, SLOT( slotPreferences() ), actionCollection(), "preferences" );
	act->setWhatsThis( i18n( "Configure kannadic." ) );
}

void frmMain::setEditEnabled( bool enabled )
{
	KAction* act;
 	for( act = actEdit.first(); act; act = actEdit.next() ){
		act->setEnabled( enabled );
	}
	actUndo->setEnabled( enabled );
	actRedo->setEnabled( enabled );
	actImport->setEnabled( enabled );
	actExport->setEnabled( enabled );
	txtFrom->setEnabled( enabled );
	txtTo->setEnabled( enabled );
	cbxType->setEnabled( enabled );
	btnOK->setEnabled( enabled );
	lvwDictionary->setEnabled( enabled );
	lvwClipboard->setEnabled( enabled );
}

void frmMain::setupStatusBar()
{
	statusBar()->insertItem( "", 0, 1, false );
	statusBar()->insertItem( i18n("Count of words:"), 1, 0, true );
	statusBar()->setSizeGripEnabled( true );
	statusBar()->show();
}

void frmMain::slotDicNew()
{
	QString dicname = KLineEditDlg::getText( i18n( "Create dictinary" ), i18n( "Input name of new dicitonary." ) );
	if( !dicname.stripWhiteSpace().isEmpty() ){
		proc = new QProcess( this );
		proc->addArgument( "mkdic" );
		proc->addArgument( dicname );
		m_dicname = dicname;
		connect( proc, SIGNAL( processExited() ), this, SLOT( slotDicNew1() ) );
		proc->start();
	}
}

void frmMain::slotDicDel()
{
	int ans = KMessageBox::warningYesNo( this, i18n("Do you delete '%1' dictionary really?").arg( m_dicname ), i18n("Delete dictionary") );
	switch( ans )
	{
		case KMessageBox::Yes:
			break;
		case KMessageBox::No:
			return;
			break;
	}
	proc = new QProcess( this );
	proc->addArgument( "kinput2ctl" );
	proc->addArgument( "stop" );
	connect( proc, SIGNAL( processExited() ), this, SLOT( slotDicDel1() ) );
	proc->start();
}

void frmMain::slotDicNew1()
{
	if( proc->normalExit() ){
		KannaDic::mkdic( m_dicname );
		slotDicOpen( m_dicname );
		proc = new QProcess( this );
		proc->addArgument( "kinput2ctl" );
		proc->addArgument( "canna" );
		proc->start();
	}
}

void frmMain::slotDicDel1()
{
	if( proc->normalExit() ){
		proc = new QProcess( this );
		proc->addArgument( "rmdic" );
		proc->addArgument( m_dicname );
		connect( proc, SIGNAL( processExited() ), this, SLOT( slotDicDel2() ) );
		proc->start();
	}
}

void frmMain::slotDicDel2()
{
	while( proc->canReadLineStderr() ){
		if( proc->readLineStderr() == QString( "Dictinary \"%1\" is in use. Cannot delete it." ).arg( m_dicname ) ){
			KMessageBox::error( this, i18n("Dictinary \"%1\" is in use. Cannot delete it.").arg( m_dicname ), i18n("Error") );
			proc = new QProcess( this );
			proc->addArgument( "kinput2ctl" );
			proc->addArgument( "canna" );
			proc->start();
			return;
		}
	}
	if( proc->normalExit() ){
		KannaDic::rmdic( m_dicname );
		m_dicname = QString::null;
		lvwDictionary->clear();
		setCaption( m_dicname );
		setEditEnabled( false );
		proc = new QProcess( this );
		proc->addArgument( "kinput2ctl" );
		proc->addArgument( "canna" );
		proc->start();
	}
}
void frmMain::slotDicOpen()
{
	if( !CheckChangedAndSaved() ) return;

	dlgOpenDic dlg( this );
	if( dlg.exec() == QDialog::Accepted )
	{
		slotDicOpen( dlg.dicname() );
	}
}

void frmMain::slotDicOpen( const QString& a_dicname )
{
	setCaption( a_dicname );
	setEditEnabled( true );
	if( m_dicname == a_dicname ){
		//return;
	}
	m_dicname = a_dicname;
	switch( tabBase->currentPageIndex() )
	{
		case TAB_WORD:
			m_blDicOpened = false;
			break;
		case TAB_LIST:
			dicOpen( a_dicname );
			m_blDicOpened = true;
			break;
	}
}

void frmMain::slotSave()
{
	Word word;
	QString line("%1 %2 %3");
	if( txtFrom->text() == "" ){
		KMessageBox::error( this, i18n("From is Empty"), i18n("Error") );
		txtFrom->setFocus();
		return;
	}
	if( txtTo->text() == "" ){
		KMessageBox::error( this, i18n("To is Empty"), i18n("Error") );
		txtTo->setFocus();
		return;
	}
	word.from = txtFrom->text();
	word.to = txtTo->text();
	word.type = cbxType->currentText();
	if( !KannaDic::addwordx( m_dicname, word ) ){
		KMessageBox::sorry( this, txtFrom->text() + "->" + txtTo->text(), i18n( "Could not add" ) );
		return;
	}
	if( m_blDicOpened ){
		KListViewItem* item = new KListViewItem( lvwDictionary, txtFrom->text(), txtTo->text(), cbxType->currentText() );
		lvwDictionary->insertItem( item );
		statusBar()->changeItem( i18n("Count of words:%1").arg( lvwDictionary->childCount() ), 1 );
	}
	txtFrom->setText( "" );
	txtTo->setText( "" );
/*
	word.to.replace( "&", "&amp;" );
	word.to.replace( "\\", "&slash;" );
	word.to.replace( " ", "&space;" );
	word.to.replace( "&slash;", "\\\\" );
	word.to.replace( "&space;", "\\ " );
	word.to.replace( "&amp;", "&" );

	word.from.replace( "&", "&amp;" );
	word.from.replace( "\\", "&slash;" );
	word.from.replace( " ", "&space;" );
	word.from.replace( "&slash;", "\\\\" );
	word.from.replace( "&space;", "\\ " );
	word.from.replace( "&amp;", "&" );

	word.type = KannaDic::name2code( QTextCodec::codecForName("eucJP")->fromUnicode(word.type) );
	proc = new QProcess( this );
	connect( proc, SIGNAL( processExited() ), this, SLOT( slotDicSave1() ) );
	proc->addArgument( "addwords" );
	proc->addArgument( m_dicname );
	kdDebug() << line.arg( word.from ).arg( word.type ).arg( word.to ) << endl;
	proc->start();
	proc->writeToStdin( line.arg( word.from ).arg( word.type ).arg( word.to ) );
	proc->closeStdin();
	proc->tryTerminate();
*/
}

void frmMain::slotDicSave()
{
	if( m_changed_index > -1 ){
		dicSave();
	}
}

void frmMain::slotDicSave1()
{
	while( proc->canReadLineStdout() ){
		kdDebug() << " " << QTextCodec::codecForName("eucJP")->fromUnicode( proc->readLineStdout() ) << endl;
	}
	while( proc->canReadLineStderr() ){
		kdDebug() << " " << QTextCodec::codecForName("eucJP")->fromUnicode( proc->readLineStderr() ) << endl;
	}
	if( proc->normalExit() ){
		if( m_blDicOpened ){
			KListViewItem* item = new KListViewItem( lvwDictionary, txtFrom->text(), txtTo->text(), cbxType->currentText() );
			lvwDictionary->insertItem( item );
			statusBar()->changeItem( i18n("Count of words:%1").arg( lvwDictionary->childCount() ), 1 );
		}
		txtFrom->setText( "" );
		txtTo->setText( "" );
	}else{
		KMessageBox::sorry( this, txtFrom->text() + "->" + txtTo->text(), i18n( "Could not add" ) );
	}
}

 bool frmMain::queryClose()
{
	return CheckChangedAndSaved();
}

void frmMain::dicOpen( QString a_dicname )
{
	QApplication::setOverrideCursor( waitCursor );

	lvwDictionary->clear();
	QValueList<Word> cd = KannaDic::catdic( m_dicname );
 	for( QValueListIterator<Word> word = cd.begin(); word != cd.end(); word++ ){
		KListViewItem* item = new KListViewItem( lvwDictionary, (*word).from, (*word).to, (*word).type );
		lvwDictionary->insertItem( item );
	}
	m_changed.clear();
	m_changed_index = m_changed.size() - 1;
	m_before.clear();
	m_after.clear();
	actRedo->setEnabled( false );
	actUndo->setEnabled( false );

	statusBar()->changeItem( i18n("Count of words:%1").arg( lvwDictionary->childCount() ), 1 );

	QApplication::restoreOverrideCursor();
}

bool frmMain::dicSave()
{
	QApplication::setOverrideCursor( waitCursor );

	Word before, after;
	KannaDic::mount( m_dicname );
	QValueList<long>::iterator it;
	int i = 0;
	for( it = m_changed.begin(); it != m_changed.end(); ++it ){
		for( int j = 0; j != *it; i++, j++ ){
			if( m_before[i].from != "" ){
				if( !KannaDic::delword( m_dicname, m_before[i] ) ){
					KMessageBox::sorry( this, m_before[i].from + "->" + m_before[i].to, i18n( "Could not delete." ) );
				}
			}
			if( m_after[i].from != "" ){
				if( !KannaDic::addword( m_dicname, m_after[i] ) ){
					KMessageBox::sorry( this, m_after[i].from + "->" + m_after[i].to, i18n( "Could not add" ) );
				}
			}
		}
	}
	KannaDic::unmount( m_dicname );
	m_changed.clear();
	m_changed_index = m_changed.size() - 1;
	m_before.clear();
	m_after.clear();
	actUndo->setEnabled( false );
	actRedo->setEnabled( false );
	statusBar()->message( i18n("Saved"), 5000 );

	QApplication::restoreOverrideCursor();
}

void frmMain::slotAppend()
{
	dlgEdit dlgedit( this );
	dlgedit.setEditMode( dlgEdit::APPEND );
	dlgedit.exec();
	if( dlgedit.result() ){
		if( isEditError( dlgedit ) ){
			dlgedit.exec();
		}else{
			Word word;
			while( m_changed.size() - m_changed_index > 1 ){
				for( int j = 0; j != m_changed[m_changed.size() - 1]; j++ ){
					m_before.pop_back();
					m_after.pop_back();
				}
				m_changed.pop_back();
			}
			m_before.push_back( word );
			word.from = dlgedit.from();
			word.to = dlgedit.to();
			word.type = dlgedit.type();
			m_after.push_back( word );
			m_changed.push_back( 1 );
			m_changed_index = m_changed.size() - 1;
			actUndo->setEnabled( true );
			actRedo->setEnabled( false );

			KListViewItem* item = new KListViewItem( lvwDictionary, word.from, word.to, word.type );
			lvwDictionary->insertItem( item );
			lvwDictionary->clearSelection();
			lvwDictionary->ensureItemVisible( item );
			lvwDictionary->setSelected( item, true );
			statusBar()->changeItem( i18n("Count of words:%1").arg( lvwDictionary->childCount() ), 1 );
		}
	}
}

void frmMain::slotEdit()
{
	Word word;
	dlgEdit dlgedit( this );
	dlgedit.setEditMode( dlgEdit::EDIT );
	QPtrList<QListViewItem> selectedItems = lvwDictionary->selectedItems();
	if( !selectedItems.count() ){
		KMessageBox::sorry( this, i18n("Select edit line") );
		return;
	}
	int last_changed = 0;
	for( QListViewItem* item = selectedItems.first(); item; item = selectedItems.next() ){
		lvwDictionary->ensureItemVisible( item );
		word.from = item->text( 0 );
		word.to = item->text( 1 );
		word.type = item->text( 2 );
		dlgedit.setFrom( word.from );
		dlgedit.setTo( word.to );
		dlgedit.setType( word.type );
		dlgedit.exec();

		while( dlgedit.result() ){
			if( isEditError( dlgedit ) ){
				dlgedit.exec();
			}else{
				while( m_changed.size() - m_changed_index > 1 ){
					for( int j = 0; j != m_changed[m_changed.size() - 1]; j++ ){
						m_before.pop_back();
						m_after.pop_back();
					}
					m_changed.pop_back();
				}
				if( word.from != dlgedit.from() || word.to != dlgedit.to() || word.type != dlgedit.type() )
				{
					m_before.push_back( word );

					word.from = dlgedit.from();
					word.to = dlgedit.to();
					word.type = dlgedit.type();
					item->setText( 0, word.from );
					item->setText( 1, word.to );
					item->setText( 2, word.type );
					m_after.push_back( word );
					last_changed++;
				}
				break;
			}
		}
		if( !dlgedit.result() ){
			break;
		}
	}
	if( last_changed ){
		m_changed.push_back( last_changed );
		m_changed_index = m_changed.size() - 1;
		actUndo->setEnabled( true );
		actRedo->setEnabled( false );
		statusBar()->changeItem( i18n("Count of words:%1").arg( lvwDictionary->childCount() ), 1 );
	}
}

void frmMain::slotDelete()
{
	QPtrList<QListViewItem> selectedItems = lvwDictionary->selectedItems();
	if( !selectedItems.count() ){
		KMessageBox::sorry( this, i18n("Select edit line(s)") );
		return;
	}
	QApplication::setOverrideCursor( waitCursor );

	while( m_changed.size() - m_changed_index > 1 ){
		for( int j = 0; j != m_changed[m_changed.size() - 1]; j++ ){
			m_before.pop_back();
			m_after.pop_back();
		}
		m_changed.pop_back();
	}

	int last_changed = 0;
	for( QListViewItem* item = selectedItems.first(); item; item = selectedItems.next() ){
		Word word;
		m_after.push_back( word );
		word.from = item->text( 0 );
		word.to = item->text( 1 );
		word.type = item->text( 2 );
		m_before.push_back( word );
		lvwDictionary->takeItem( item );
		last_changed++;
		actUndo->setEnabled( true );
		actRedo->setEnabled( false );
	}
	if( last_changed ){
		m_changed.push_back( last_changed );
		m_changed_index = m_changed.size() - 1;
		actUndo->setEnabled( true );
		actRedo->setEnabled( false );
		statusBar()->changeItem( QString( i18n("Count of words:%1") ).arg( lvwDictionary->childCount() ), 1 );
	}

	QApplication::restoreOverrideCursor();
}

void frmMain::slotCut()
{
	QPtrList<QListViewItem> selectedItems = lvwDictionary->selectedItems();
	if( !selectedItems.count() ){
		KMessageBox::sorry( this, i18n("Select cut line(s)") );
		return;
	}
	QApplication::setOverrideCursor( waitCursor );

	lvwClipboard->clear();

	while( m_changed.size() - m_changed_index > 1 ){
		for( int j = 0; j != m_changed[m_changed.size() - 1]; j++ ){
			m_before.pop_back();
			m_after.pop_back();
		}
		m_changed.pop_back();
	}

	int last_changed = 0;
	QListViewItemIterator it( lvwDictionary );
	for( QListViewItem* item = selectedItems.first(); item; item = selectedItems.next() ){
		Word word;
		m_after.push_back( word );
		word.from = item->text( 0 );
		word.to = item->text( 1 );
		word.type = item->text( 2 );
		m_before.push_back( word );
		lvwClipboard->insertItem( (KListViewItem*)item );
		last_changed++;
	}
	if( last_changed ){
		m_changed.push_back( last_changed );
		m_changed_index = m_changed.size() - 1;
		actUndo->setEnabled( true );
		actRedo->setEnabled( false );
		statusBar()->changeItem( QString( i18n("Count of words:%1") ).arg( lvwDictionary->childCount() ), 1 );
	}

	QApplication::restoreOverrideCursor();
}

void frmMain::slotCopy()
{
	QPtrList<QListViewItem> selectedItems = lvwDictionary->selectedItems();
	if( !selectedItems.count() ){
		KMessageBox::sorry( this, i18n("Select copy line(s)") );
		return;
	}
	QApplication::setOverrideCursor( waitCursor );

	lvwClipboard->clear();
	QListViewItemIterator it( lvwDictionary );
	for( QListViewItem* item = selectedItems.first(); item; item = selectedItems.next() ){
		KListViewItem* newitem = new KListViewItem( lvwClipboard, item->text( 0 ), item->text( 1 ), item->text( 2 ) );
		lvwClipboard->insertItem( newitem );
	}

	QApplication::restoreOverrideCursor();
}

void frmMain::slotPaste()
{
	QApplication::setOverrideCursor( waitCursor );

	lvwDictionary->clearSelection();

	while( m_changed.size() - m_changed_index > 1 ){
		for( int j = 0; j != m_changed[m_changed.size() - 1]; j++ ){
			m_before.pop_back();
			m_after.pop_back();
		}
		m_changed.pop_back();
	}

	int last_changed = 0;
	QListViewItemIterator it( lvwClipboard );
	KListViewItem* newitem;
	while ( it.current() ) {
		Word word;
		m_before.push_back( word );
		word.from = it.current()->text( 0 );
		word.to = it.current()->text( 1 );
		word.type = it.current()->text( 2 );
		m_after.push_back( word );
		last_changed++;

		newitem = new KListViewItem( lvwDictionary, word.from, word.to, word.type );
		lvwDictionary->insertItem( newitem );
		lvwDictionary->setSelected( newitem, true );
		++it;
	}
	lvwDictionary->ensureItemVisible( newitem );

	if( last_changed ){
		m_changed.push_back( last_changed );
		m_changed_index = m_changed.size() - 1;
		actUndo->setEnabled( true );
		actRedo->setEnabled( false );
		statusBar()->changeItem( QString( i18n("Count of words:%1") ).arg( lvwDictionary->childCount() ), 1 );
	}
	QApplication::restoreOverrideCursor();
}

void frmMain::slotUndo()
{
	QApplication::setOverrideCursor( waitCursor );

	lvwDictionary->clearSelection();
	Word word;
	int i = 0;
	QValueList<long>::iterator it;
	for( it = m_changed.begin(); it != m_changed.at( m_changed_index ); ++it ){
		i += *it;
	}
	KListViewItem* newitem;
	for( int j = 0; j != m_changed[m_changed_index]; j++ ){
		if( m_after[i+j].from != "" ){
			word = m_after[i+j];
			QListViewItemIterator it( lvwDictionary );
			while ( it.current() ) {
				if( word.from == it.current()->text( 0 ) && word.to == it.current()->text( 1 ) && word.type == it.current()->text( 2 ) ){
					lvwDictionary->takeItem( it.current() );
					break;
				}
				++it;
			}
		}
		if( m_before[i+j].from != ""  ){
			word = m_before[i+j];
			newitem = new KListViewItem( lvwDictionary, word.from, word.to, word.type );
			lvwDictionary->insertItem( newitem );
			lvwDictionary->setSelected( newitem, true );
		}
	}
	lvwDictionary->ensureItemVisible( newitem );
	m_changed_index--;
	actUndo->setEnabled( m_changed_index != -1 );
	actRedo->setEnabled( true );
	statusBar()->changeItem( i18n("Count of words:%1").arg( lvwDictionary->childCount() ), 1 );

	QApplication::restoreOverrideCursor();
}

void frmMain::slotRedo()
{
	QApplication::setOverrideCursor( waitCursor );

	lvwDictionary->clearSelection();

	Word word;
	int i = 0;
	m_changed_index++;
	QValueList<long>::iterator it;
	for( it = m_changed.begin(); it != m_changed.at( m_changed_index ); ++it ){
		i += *it;
	}
	KListViewItem* newitem;
	for( int j = 0; j != m_changed[m_changed_index]; j++ ){
		if( m_before[i+j].from != ""  ){
			word = m_before[i+j];
			QListViewItemIterator it( lvwDictionary );
			while ( it.current() ) {
				if( word.from == it.current()->text( 0 ) && word.to == it.current()->text( 1 ) && word.type == it.current()->text( 2 ) ){
					lvwDictionary->takeItem( it.current() );
					break;
				}
				++it;
			}
		}
		if( m_after[i+j].from != "" ){
			word = m_after[i+j];
			newitem = new KListViewItem( lvwDictionary, word.from, word.to, word.type );
			lvwDictionary->insertItem( newitem );
			lvwDictionary->setSelected( newitem, true );
		}
	}
	lvwDictionary->ensureItemVisible( newitem );
	actUndo->setEnabled( true );
	actRedo->setEnabled( m_changed_index != m_changed.size() - 1 );
	statusBar()->changeItem( QString( i18n("Count of words:%1") ).arg( lvwDictionary->childCount() ), 1 );

	QApplication::restoreOverrideCursor();
}

void frmMain::slotSelectAll()
{
	lvwDictionary->selectAll( true );
}

void frmMain::slotDeselect()
{
	lvwDictionary->selectAll( false );
}

void frmMain::importTextDictionary( QValueList<Word> list )
{
	if( list.size() == 0 ) return;

	QApplication::setOverrideCursor( waitCursor );

	int last_changed = 0;
	while( m_changed.size() - m_changed_index > 1 ){
		for( int j = 0; j != m_changed[m_changed.size() - 1]; j++ ){
			m_before.pop_back();
			m_after.pop_back();
		}
		m_changed.pop_back();
	}
 	for( QValueListIterator<Word> word = list.begin(); word != list.end(); word++ ){
		KListViewItem* item = new KListViewItem( lvwDictionary, (*word).from, (*word).to, (*word).type );
		lvwDictionary->insertItem( item );

		Word add, empty;
		m_before.push_back( empty );
		add.from = (*word).from;
		add.to = (*word).to;
		add.type = (*word).type;
		m_after.push_back( add );
		last_changed++;
	}

	if( last_changed ){
		m_changed.push_back( last_changed );
		m_changed_index = m_changed.size() - 1;
		actRedo->setEnabled( false );
		actUndo->setEnabled( true );
		statusBar()->changeItem( i18n("Count of words:%1").arg( lvwDictionary->childCount() ), 1 );
	}

	QApplication::restoreOverrideCursor();
}

void frmMain::slotImportCannaTextDictionary()
{
	importTextDictionary( Importer::importCannaTextDictionary( this ) );
}

void frmMain::slotImportWindowsTextDictionary()
{
	importTextDictionary( Importer::importWindowsTextDictionary( this ) );
}

void frmMain::slotImportKannadicXMLDictionary()
{
	importTextDictionary( Importer::importKannadicXMLDictionary( this ) );
}

void frmMain::slotExportKannadicXMLDictionary()
{
	Exporter::exportKannadicXMLDictionary( this, lvwDictionary );
}

void frmMain::slotShowClipboard( bool bl )
{
	actShowClipboard->setChecked( bl );
	if( bl ){
		clipboard->show();
	}else{
		clipboard->hide();
	}
}

void frmMain::slotShowMenubar( bool bl )
{
	actShowMenubar->setChecked( bl );
	if( bl ){
		menuBar()->show();
	}else{
		menuBar()->hide();
	}
}

void frmMain::slotShowToolbar( bool bl )
{
	actShowToolbar->setChecked( bl );
	if( bl ){
		toolBar()->show();
	}else{
		toolBar()->hide();
	}
}

void frmMain::slotShowStatusbar( bool bl )
{
	actShowStatusbar->setChecked( bl );
	if( bl ){
		statusBar()->show();
	}else{
		statusBar()->hide();
	}
}

void frmMain::slotPreferences()
{
	dlgPreferences dlg( this );
	dlg.setColor( lvwDictionary->alternateBackground() );
	dlg.setSaveClipboard( m_saveClipboard );
	if( dlg.exec() == QDialog::Accepted )
	{
		lvwDictionary->setAlternateBackground( dlg.color() );
		m_saveClipboard = dlg.saveClipboard();

		// to repaint...
		lvwDictionary->hide();
		lvwDictionary->show();
	}
}

void frmMain::slotPopup( QListViewItem* a_item, const QPoint& a_pos, int a_column )
{
	if( a_item ){
		( (KPopupMenu*)( factory()->container( "item_popup", this ) ) )->exec( a_pos );
	}else{
		( (KPopupMenu*)( factory()->container( "noitem_popup", this ) ) )->exec( a_pos );
	}
}

void frmMain::slotClipboardHide()
{
	slotShowClipboard( false );
}

bool frmMain::isEditError( dlgEdit& dlgedit )
{
	if( dlgedit.from() == "" ){
		KMessageBox::error( this, i18n("From is Empty"), i18n("Error") );
		return true;
	}
	if( dlgedit.to() == "" ){
		KMessageBox::error( this, i18n("To is Empty"), i18n("Error") );
		return true;
	}
	return false;
}

bool frmMain::CheckChangedAndSaved()
{
	bool blRtn = true;
	if( m_changed_index > -1 ){
		int ans = KMessageBox::questionYesNoCancel( this, i18n("This dictionary is changed.\nDo you save change?"), i18n("Comfirm") );
		switch( ans )
		{
			case KMessageBox::Yes:
				blRtn = dicSave();
				break;
			case KMessageBox::No:
				break;
			case KMessageBox::Cancel:
				blRtn = false;
				break;
		}
	}
	return blRtn;
}

void frmMain::loadClipboard()
{
	QString filename = locateLocal( "data", "kannadic/clipboard.ktd" );
	QValueList<Word> list = Importer::importKannadicXMLDictionary( this, KURL( filename ) );

	if( list.size() == 0 ) return;

	QApplication::setOverrideCursor( waitCursor );

 	for( QValueListIterator<Word> word = list.begin(); word != list.end(); word++ ){
		KListViewItem* item = new KListViewItem( lvwDictionary, (*word).from, (*word).to, (*word).type );
		lvwClipboard->insertItem( item );
	}

	QApplication::restoreOverrideCursor();
}

void frmMain::saveClipboard()
{
	QString filename = locateLocal( "data", "kannadic/clipboard.ktd" );
	KURL url;
	url.setPath( filename );
	Exporter::exportKannadicXMLDictionary( this, lvwClipboard, url );
}

void frmMain::slotDictionaryDropped( KListView* list, QDropEvent* e, QListViewItem* item )
{
	kdDebug() << item->text(0) << endl;
}
