#include "kannadic.h"
#include "type.h"
#include <klocale.h>
#include <kdebug.h>
#include <stdio.h>
#include <pwd.h>
#include <unistd.h>
#include <stdlib.h>
#include <qtextcodec.h>
#include <qtextstream.h>
#include <qmap.h>
#include <qfile.h>
#include <qdir.h>
#include <qregexp.h>
#include <qstring.h>
#include <kfiledialog.h>

#define BUFSIZE 1024

using namespace std;

//#include <canna/RK.h>
extern "C"{
	int RkGetWordTextDic( int, const char*, const char*, char*, int );
	int RkInitialize( char* );
	int RkFinalize( void );
	int RkMountDic( int, char*, int );
	int RkDefineDic( int, char*, char* );
	int RkDeleteDic( int, char*, char* );
	char* RkwGetServerName( void );
}

QValueList<Word> KannaDic::catdic( QString& a_dicname )
{
	QValueList<Word> list;
	QString dirname;
	char buf[BUFSIZE];
	int ret;

	QTextCodec* codec = QTextCodec::codecForName("eucJP");

	dirname = QString(":user/");
	dirname += searchuname();

	ret = RkInitialize( servername() );
	if( ret < 0 ){
		kdDebug() << "catdic error " << a_dicname << endl;
		return list;
	}
	ret = RkGetWordTextDic( 0, dirname, codec->fromUnicode( a_dicname ), buf, BUFSIZE-1 );
	if( ret < 0 ){
		kdDebug() << "catdic error " << a_dicname << endl;
		return list;
	}
	int i = 0;
	while( ret > 0 ){
		QString str( escape(buf) );
		QTextStream line( str, IO_ReadOnly );
		Word word;
		line >> word.from;
		word.from = codec->toUnicode( unescape(word.from) );
		while( !line.atEnd() ){
			QString temp, type;
			line >> temp;
			type = code2name( unescape(temp) );
			if( type != QString::null ){
				word.type = codec->toUnicode( type );
			}else{
				word.to = codec->toUnicode( unescape( temp ) );
				list.push_back( word );
			}
		}
		ret = RkGetWordTextDic( 0, "", "", buf, BUFSIZE-1 );
		i++;
//kdDebug() << codec->toUnicode( str ) << endl;
	}
//kdDebug() << list.size() << endl;
	RkFinalize();
	return list;
}

bool KannaDic::mount( QString& a_dicname )
{
	int ret;
	char dicname[BUFSIZE];

	ret = RkInitialize( servername() );
	if( ret < 0 ){
		kdDebug() << QString( "mount initialize error %1" ).arg( ret ) << endl;
		return false;
	}
	QTextCodec* codec = QTextCodec::codecForName("eucJP");
	sprintf( dicname, codec->fromUnicode( a_dicname ) );
	ret = RkMountDic( 0, dicname, 0 );
	if( ret < 0 ){
		kdDebug() << QString( "mount mount error %1" ).arg( ret ) << endl;
		RkFinalize();
		return false;
	}
	return true;
}

bool KannaDic::addword( QString& a_dicname, Word& a_word )
{
	int ret;
	char dicname[BUFSIZE], word[BUFSIZE];

	QTextCodec* codec = QTextCodec::codecForName("eucJP");
	sprintf( dicname, codec->fromUnicode( a_dicname ) );

	QString from, to, type;
	from = codec->fromUnicode( a_word.from );
	type = name2code( codec->fromUnicode( a_word.type ) );

	to = codec->fromUnicode( a_word.to );
	to.replace( "&", "&amp;" );
	to.replace( "\\", "&slash;" );
	to.replace( " ", "&space;" );
	to.replace( "&slash;", "\\\\" );
	to.replace( "&space;", "\\ " );
	to.replace( "&amp;", "&" );

	sprintf( word, QString(from + " " + type + " " + to ) );

	ret = RkDefineDic( 0, dicname, word );
	if( ret < 0 ){
		kdDebug() << "addword define error " << word << endl;
		unmount( a_dicname );
		mount( a_dicname );
		return false;
	}
	return true;
}

bool KannaDic::delword( QString& a_dicname, Word& a_word )
{
	int ret;
	char dicname[BUFSIZE], word[BUFSIZE];

	QTextCodec* codec = QTextCodec::codecForName("eucJP");
	sprintf( dicname, codec->fromUnicode( a_dicname ) );

	QString from, to, type;
	from = codec->fromUnicode( a_word.from );

	type = name2code( codec->fromUnicode( a_word.type ) );

	to = codec->fromUnicode( a_word.to );
	to.replace( "&", "&amp;" );
	to.replace( "\\", "&slash;" );
	to.replace( " ", "&space;" );
	to.replace( "&slash;", "\\\\" );
	to.replace( "&space;", "\\ " );
	to.replace( "&amp;", "&" );

	sprintf( word, QString( from + " " + type + " " + to ) );

	ret = RkDeleteDic( 0, dicname, word );
	if( ret < 0 ){
		kdDebug() << "delword delete error " << word << endl;
		unmount( a_dicname );
		mount( a_dicname );
		return false;
	}
	return true;
}

void KannaDic::unmount( QString& a_dicname )
{
	RkFinalize();
}

bool KannaDic::addwordx( QString& a_dicname, Word& a_word )
{
	int ret;
	char dicname[BUFSIZE], word[BUFSIZE];

	ret = RkInitialize( servername() );
	if( ret < 0 ){
		kdDebug() << QString( "addword initialize error %1").arg( ret ) << endl;
		return false;
	}

	QTextCodec* codec = QTextCodec::codecForName("eucJP");
	sprintf( dicname, codec->fromUnicode( a_dicname ) );

	QString from, to, type;
	from = codec->fromUnicode( a_word.from );
	type = name2code( codec->fromUnicode( a_word.type ) );

	to = codec->fromUnicode( a_word.to );
	to.replace( "&", "&amp;" );
	to.replace( "\\", "&slash;" );
	to.replace( " ", "&space;" );
	to.replace( "&slash;", "\\\\" );
	to.replace( "&space;", "\\ " );
	to.replace( "&amp;", "&" );

	sprintf( word, QString(from + " " + type + " " + to ) );

	ret = RkMountDic( 0, dicname, 0 );
	if( ret < 0 ){
		kdDebug() << "addword mount error " << ret << endl;
		RkFinalize();
		return false;
	}
	ret = RkDefineDic( 0, dicname, word );
	if( ret < 0 ){
		kdDebug() << "addword define error " << word << endl;
		RkFinalize();
		return false;
	}
	RkFinalize();
	return true;
}

bool KannaDic::mkdic( QString& a_dicname )
{
	QString dotCanna( KannaDic::dotCanna() );
	if( dotCanna.isEmpty() ){
		return false;
	}
	QFile fin( dotCanna );
	if ( !fin.open( IO_ReadOnly ) ){
		kdDebug() << "open false" << endl;
		return false;
	}
	dotCanna = "";
	QTextStream in( &fin );
	bool isDic = false;
	QTextCodec* codec = QTextCodec::codecForName("eucJP");
	while( !in.atEnd() ){
		QString str( in.readLine() );
		if( isDic && str == " )" ){
			dotCanna +=  QString(" :user \"%1\"\n").arg( a_dicname );
//			dotCanna +=  QString(" :user \"%1\"\n").arg( codec->fromUnicode( a_dicname ) );
			isDic = false;
		}
		if( str == "(use-dictionary" ){
			isDic = true;
		}
		dotCanna += str + "\n";
	}
	fin.close();

	QFile fout( QDir::home().absPath() + "/.canna" );
	if ( !fout.open( IO_WriteOnly ) ){
		kdDebug() << "open false" << endl;
		return false;
	}
	QTextStream out( &fout );
	out << dotCanna;
	fout.close();
	return true;
}

bool KannaDic::rmdic( QString& a_dicname )
{
	QString dotCanna( KannaDic::dotCanna() );
	if( dotCanna.isEmpty() ){
		return false;
	}
	QFile fin( dotCanna );
	if ( !fin.open( IO_ReadOnly ) ){
		kdDebug() << "open false" << endl;
		return false;
	}
	dotCanna = "";
	QTextStream in( &fin );
	bool isDic = false;
	QString dicname = QTextCodec::codecForName("eucJP")->fromUnicode( a_dicname );
	while( !in.atEnd() ){
		QString str( in.readLine() );
		if( isDic && str == " )" ){
			isDic = false;
		}
		if( isDic ){
			QRegExp regexp( ":user +\"(.*)\"", false, false );
			if( regexp.search( str ) != -1 ){
				if( regexp.cap(1) == a_dicname ){
					continue;
				}
			}
		}
		if( str == "(use-dictionary" ){
			isDic = true;
		}
		dotCanna += str + "\n";
	}
	fin.close();

	QFile fout( QDir::home().absPath() + "/.canna" );
	if ( !fout.open( IO_WriteOnly ) ){
		kdDebug() << "open false" << endl;
		return false;
	}
	QTextStream out( &fout );
	out << dotCanna;
	fout.close();
	return true;
}

QString KannaDic::dotCanna()
{
	if( QFile::exists( QDir::home().absPath() + "/.canna" ) ){
		return QDir::home().absPath() + "/.canna";
	}else if( QFile::exists( "/usr/local/lib/canna/default.canna" ) ){
		return "/usr/local/lib/canna/default.canna";
	}else if( QFile::exists( "/usr/lib/canna/default.canna" ) ){
		return "/usr/lib/canna/default.canna";
	}else if( QFile::exists( "/var/lib/canna/default.canna" ) ){
		return "/var/lib/canna/default.canna";
	}else{
		return KFileDialog::getOpenFileName( QString::null, I18N_NOOP( "*.canna|canna configuration file" ) );
	}
}

QStringList KannaDic::type()
{
	QStringList list;
	QTextCodec* codec = QTextCodec::codecForName("eucJP");

	int count = sizeof(code_name)/sizeof(struct Type);
	for( int i = 0; i != count; i++ ){
		list.push_back( codec->toUnicode( code_name[i].name ) );
	}
	return list;
}

QString KannaDic::name2code( QString name )
{
	static QMap<QString, QString> table;
	if( table.empty() ){
		int count = sizeof(code_name)/sizeof(struct Type);
		for( int i = 0; i != count; i++ ){
			table[code_name[i].name] = code_name[i].code;
		}
	}
	return table[name];
}

QString KannaDic::name2codex( QString name )
{
	int count = sizeof(code_name)/sizeof(struct Type);
	for( int i = 0; i != count; i++ ){
		if( code_name[i].name == name ){
			return code_name[i].code;
		}
	}
	return QString::null;
}

QString KannaDic::code2name( QString code )
{
	static QMap<QString, QString> table;
	if( table.empty() ){
		int count = sizeof(code_name)/sizeof(struct Type);
		for( int i = 0; i != count; i++ ){
			table[code_name[i].code] = code_name[i].name;
		}
	}
	return table[code];
}

QString KannaDic::code2namex( QString code )
{
	int count = sizeof(code_name)/sizeof(struct Type);
	for( int i = 0; i != count; i++ ){
		if( code_name[i].code == code ){
			return code_name[i].name;
		}
	}
	return QString::null;
}

char* KannaDic::searchuname()
{
	char *username = NULL;

	struct passwd *pass = getpwuid( getuid() ) ;
	if ( pass ) username = pass->pw_name ;

	if ( username == NULL ) {
		if( (username = getlogin()) == NULL ) {
		if( (username = getenv( "LOGNAME" )) == NULL ) {
			username = getenv( "USER" ) ;
		}
		}
	}
	if ( username == NULL ) {
		exit(-2);
	}
	return username;
}

char* KannaDic::servername()
{
	return RkwGetServerName();
//	return getenv( "CANNA_SERVER" );
}

QString KannaDic::escape( QString from )
{
	QString to( from );
	to.replace( "&", "&amp;" );
	to.replace( "\\\\", "&slash;" );
	to.replace( "\\ ", "&space;" );
	return to;

}

QString KannaDic::unescape( QString from )
{
	QString to( from );
	to.replace( "&slash;", "\\" );
	to.replace( "&space;", " " );
	to.replace( "&amp;", "&" );
	return to;

}
